﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace VolumeRayCaster
{
    class Matrix
    {
        public Matrix()
        {
            vals = new double[4,3];
        }
        public static Matrix operator *(Matrix a, Matrix b)
        {
            Matrix r = new Matrix();
            for (int i = 0; i < 4; ++i)
            {
                for (int k = 0; k < 3; ++k)
                {
                    for (int j = 0; j < 3; ++j)
                    {
                        r.vals[i, j] += a.vals[i, k] * b.vals[k, j];
                    }
                }
            }
            r.vals[3, 0] += b.vals[3, 0];
            r.vals[3, 1] += b.vals[3, 1];
            r.vals[3, 2] += b.vals[3, 2];
            return r;
        }
        public double this[int i, int j]
        {
            get
            {
                return vals[i, j];
            }
        }
        protected double[,] vals;
    }

    class IdentityMatrix : Matrix
    {
        public IdentityMatrix()
            : base()
        {
            vals[0, 0] = 1.0;
            vals[1, 1] = 1.0;
            vals[2, 2] = 1.0;
        }
    }

    class TranslationMatrix : IdentityMatrix
    {
        public TranslationMatrix(double x, double y, double z)
            : base()
        {
            vals[3, 0] = x;
            vals[3, 1] = y;
            vals[3, 2] = z;
        }
    }
    class XRotationMatrix : IdentityMatrix
    {
        public XRotationMatrix(double angle)
            : base()
        {
            vals[1, 1] = vals[2, 2] = Math.Cos(angle);
            vals[2, 1] = Math.Sin(angle);
            vals[1, 2] = -vals[2, 1];
        }
    }
    class YRotationMatrix : IdentityMatrix
    {
        public YRotationMatrix(double angle)
            : base()
        {
            vals[0, 0] = vals[2, 2] = Math.Cos(angle);
            vals[0, 2] = Math.Sin(angle);
            vals[2, 0] = -vals[0, 2];
        }
    }
    class ZRotationMatrix : IdentityMatrix
    {
        public ZRotationMatrix(double angle)
            : base()
        {
            vals[0, 0] = vals[1, 1] = Math.Cos(angle);
            vals[1, 0] = Math.Sin(angle);
            vals[0, 1] = -vals[1, 0];
        }
    }
    class ZXYRotationMatrix
    {
        static public Matrix New(double z, double x, double y)
        {
            Matrix zm = new ZRotationMatrix(z);
            Matrix xm = new XRotationMatrix(x);
            Matrix ym = new YRotationMatrix(y);
            return zm * xm * ym;
        }
    }
}
