﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace VolumeRayCaster
{
    // Wektor trójwymiarowy, który w rzeczywistości jest czterowymiarowej postaci (x,y,z,1),
    // żeby można było go translować czterowierszowymi macierzami
    class Vector
    {
        public Vector()
        {
        }
        public Vector(double x, double y, double z)
        {
            this.x = x;
            this.y = y;
            this.z = z;
        }
        public static Vector operator +(Vector a, Vector b)
        {
            return new Vector(a.x + b.x, a.y + b.y, a.z + b.z);
        }
        public static Vector operator -(Vector a, Vector b)
        {
            return new Vector(a.x - b.x, a.y - b.y, a.z - b.z);
        }
        public static double operator *(Vector a, Vector b)
        {
            return a.x*b.x + a.y*b.y + a.z*b.z;
        }
        public static Vector operator *(Vector v, Matrix m)
        {
            Vector r = new Vector();
            r.x = v.x * m[0, 0] + v.y * m[1, 0] + v.z * m[2, 0] + m[3, 0];
            r.y = v.x * m[0, 1] + v.y * m[1, 1] + v.z * m[2, 1] + m[3, 1];
            r.z = v.x * m[0, 2] + v.y * m[1, 2] + v.z * m[2, 2] + m[3, 2];
            return r;
        }
        public static Vector JuliaMult(Vector a, Vector b)
        {
            return new Vector(  a.x*b.x - a.y*b.y - a.z*b.z,
                                a.x*b.y + a.y*b.x - a.y*b.z - a.z*b.y,
                                a.x*b.z + a.z*b.x);
        }
        public static Vector Cross(Vector a, Vector b)
        {
            return new Vector(a.y * b.z - a.z * b.y, a.z * b.x - a.x * b.z, a.x * b.y - a.y * b.x);
        }
        public static Vector operator *(double a, Vector v)
        {
            return new Vector(a * v.x, a * v.y, a * v.z);
        }
        public static Vector operator *(Vector v, double a)
        {
            return new Vector(a * v.x, a * v.y, a * v.z);
        }
        public static Vector operator-(Vector v)
        {
            return new Vector(-v.x, -v.y, -v.z);
        }
        public double Length
        {
            get
            {
                return Math.Sqrt(x * x + y * y + z * z);
            }
        }
        public double LengthSqr
        {
            get
            {
                return x * x + y * y + z * z;
            }
        }
        public double Min
        {
            get
            {
                if (x < y)
                {
                    if (x < z) return x;
                    return z;
                }
                if (y < z) return y;
                return z;
            }
        }
        public double Max
        {
            get
            {
                if (x < y)
                {
                    if (y < z) return z;
                    return y;
                }
                if (x < z) return z;
                return x;
            }
        }
        public void Normalize()
        {
            double lenInv = 1 / Length;
            x *= lenInv;
            y *= lenInv;
            z *= lenInv;
        }
        public double x, y, z;
    }

    // Wektor, który przemnożony przez macierz nie transluje się
    // (x,y,z,0)
    class DirectionVector : Vector
    {
        public static DirectionVector operator *(DirectionVector v, Matrix m)
        {
            DirectionVector r = new DirectionVector();
            r.x = v.x * m[0, 0] + v.y * m[1, 0] + v.z * m[2, 0];
            r.y = v.x * m[0, 1] + v.y * m[1, 1] + v.z * m[2, 1];
            r.z = v.x * m[0, 2] + v.y * m[1, 2] + v.z * m[2, 2];
            return r;
        }
    }
}
